/*
   This program parses and then evaluates strings from Language_7.

   These examples demonstrate some unusual ways in which we can use apply.
*/

public class Language_7_Examples_4
{
   public static void main(String[] args)
   {
      String[] programs =
      {
         "( prog " +
           "( var a 2 ) " +
           "( fun f ( lambda x ( + x a ) ) ) " +
           "( var r1 ( apply f 3 ) ) )",  // returns 5
      };

      int i = 0;
      for (i = 0; i < programs.length; i++)
      {
         System.out.println(i + " =============================================");

         // Print the expression.
         //System.out.println( programs[i] + "\n" );

         // Build and evaluate the AST that represents the expression.
         try
         {
            Tokenizer tokens = new Tokenizer(programs[i]);

            // Print the token list.
            System.out.println( tokens + "\n" );

            Tree ast = BuildTree.buildTree(tokens);

            // Print the AST as an S-expression
            System.out.println( ast + "\n" );

            // Pretty-print the abstract syntax tree.
            System.out.println( PrettyPrinter.prettyPrint( ast ) + "\n" );

            // Print the infix version of the expression.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            // Create dot and png files from the AST.
            if(true)
            try
            {
               // Create the (empty) dot file.
               String baseName = String.format("Language_7_3.%02d", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                            new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println( Tree2dot_ver1.tree2dot(ast) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println( e );
            }

            // Evaluate the expression (interpret the AST).
            Value value = Evaluate.eval( ast );
            System.out.println( "result = " + value + "\n" );
         }
         catch (ParseException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
         catch (EvalException e)
         {
            System.out.println(e);
          //e.printStackTrace();
         }
      }
   }//main()
}
